@@ -13,24 +13,15 @@
 #    under the License.
 
 import os
+import subprocess
+
+_giturl = 'https://git.openstack.org/cgit/{}/tree/doc/source'
+_html_context_data = None
 
 
 def builder_inited(app):
     theme_dir = os.path.join(os.path.dirname(__file__), 'theme')
     app.info('Using openstackdocstheme Sphinx theme from %s' % theme_dir)
-    # Insert our theme directory at the front of the search path and
-    # force the theme setting to use the one in the package. This is
-    # done here, instead of in setup(), because conf.py is read after
-    # setup() runs, so if the conf contains these values the user
-    # values overwrite these. That's not bad for the theme, but it
-    # breaks the search path.
-    app.config.html_theme_path.insert(0, theme_dir)
-    # Set the theme name
-    app.config.html_theme = 'openstackdocstheme'
-    # Re-initialize the builder, if it has the method for setting up
-    # the templates and theme.
-    if hasattr(app.builder, 'init_templates'):
-        app.builder.init_templates()
 
 
 def get_pkg_path():
@@ -54,5 +45,33 @@ def get_openstack_logo_path():
     return os.path.join(get_pkg_path(), *args)
 
 
+def _html_page_context(app, pagename, templatename, context, doctree):
+    global _html_context_data
+    if _html_context_data is None:
+        _html_context_data = {}
+        _html_context_data['gitsha'] = subprocess.check_output(
+            ['git', 'rev-parse', 'HEAD'],
+        ).decode('utf-8').strip()
+        repo_name = app.config.repository_name
+        if repo_name:
+            _html_context_data['giturl'] = _giturl.format(repo_name)
+        bug_project = app.config.bug_project
+        if bug_project:
+            _html_context_data['bug_project'] = bug_project
+        bug_tag = app.config.bug_tag
+        if bug_tag:
+            _html_context_data['bug_tag'] = bug_tag
+    context.update(_html_context_data)
+
+
 def setup(app):
+    app.info('connecting events for openstackdocstheme')
     app.connect('builder-inited', builder_inited)
+    app.connect('html-page-context', _html_page_context)
+    app.add_config_value('repository_name', '', 'env')
+    app.add_config_value('bug_project', '', 'env')
+    app.add_config_value('bug_tag', '', 'env')
+    app.add_html_theme(
+        'openstackdocs',
+        os.path.abspath(os.path.dirname(__file__)) + '/theme/openstackdocs',
+    )
